"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const IS_DEV = process.env.NODE_ENV === 'development';
function log(action, path, message) {
    if (IS_DEV) {
        console.log(`[API - ${action}] ${message}: ${path}`);
    }
}
class API {
    constructor(baseURL, headers) {
        this.baseURL = baseURL;
        this.headers = headers || {};
        this.cache = new Map();
    }
    query(name, variables) {
        const params = new URLSearchParams({
            name,
            variables: JSON.stringify(variables || {}),
        });
        const url = this.baseURL + '/query?' + params.toString();
        if (this.cache.has(url)) {
            log('query', url, 'Loading from cache');
            return Promise.resolve(this.cache.get(url));
        }
        log('query', url, 'Fetching');
        return fetch(url, {
            method: 'GET',
            credentials: 'include',
            headers: Object.assign({ 'Content-Type': 'application/json' }, this.headers),
        })
            .then((response) => {
            if (response.ok) {
                return response.json();
            }
            throw new Error('Server Error');
        })
            .then((data) => {
            if (this.cache.size > 30) {
                const key = this.cache.keys().next().value;
                this.cache.delete(key);
            }
            this.cache.set(url, data);
            return data;
        });
    }
    mutate(name, variables) {
        const params = new URLSearchParams({
            name,
        });
        return fetch(this.baseURL + '/mutate?' + params.toString(), {
            method: 'POST',
            credentials: 'include',
            headers: Object.assign({ 'Content-Type': 'application/json' }, this.headers),
            body: JSON.stringify(variables),
        }).then((response) => response.json());
    }
}
const apiPlugin = (context, inject) => {
    var _a;
    const namespace = "<%= options.namespace || '' %>";
    let baseURL = namespace;
    if (process.server) {
        baseURL = 'http://0.0.0.0:3000' + namespace;
    }
    inject('graphql', new API(baseURL, (_a = context.req) === null || _a === void 0 ? void 0 : _a.headers));
};
exports.default = apiPlugin;
