var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import path from 'path';
import fs from 'fs';
import mkdirp from 'mkdirp';
import chokidar from 'chokidar';
import consola from 'consola';
import serverMiddleware from './serverMiddleware';
import graphqlImport from './graphqlImport';
const logger = consola.withTag('nuxt-graphql-middleware');
const PLUGIN_PATH = path.resolve(__dirname, 'plugin.js');
var FileType;
(function (FileType) {
    FileType["Query"] = "query";
    FileType["Mutation"] = "mutation";
})(FileType || (FileType = {}));
function resolveGraphqlFile(file, resolver) {
    return fs.promises
        .readFile(file)
        .then((buffer) => buffer.toString())
        .then((source) => graphqlImport(source, resolver));
}
function writeSource(dest, filePath, source) {
    const fileName = path.basename(filePath);
    const out = path.resolve(dest, fileName);
    return fs.promises.writeFile(out, source);
}
function resolveGraphql(files, map, resolver, filesMap, type, outputPath) {
    return Promise.all(Object.keys(files).map((name) => {
        const filePath = files[name];
        const file = resolver(filePath);
        return resolveGraphqlFile(file, resolver).then((source) => {
            map.set(name, source);
            if (outputPath) {
                writeSource(outputPath, filePath, source);
            }
            filesMap.set(file, {
                type,
                name,
                file,
            });
        });
    }));
}
const graphqlMiddleware = function () {
    return __awaiter(this, void 0, void 0, function* () {
        const options = this.options;
        const provided = (this.options.graphqlMiddleware ||
            {});
        const config = {
            graphqlServer: provided.graphqlServer || '',
            endpointNamespace: provided.endpointNamespace || '/__graphql_middleware',
            debug: provided.debug || options.dev,
            queries: provided.queries || {},
            mutations: provided.mutations || {},
            outputPath: provided.outputPath || '',
            server: provided.server,
        };
        const resolver = this.nuxt.resolver.resolvePath;
        this.addPlugin({
            src: PLUGIN_PATH,
            options: {
                namespace: config.endpointNamespace,
            },
        });
        const fileMap = new Map();
        const queries = new Map();
        const mutations = new Map();
        const outputPath = config.outputPath ? resolver(config.outputPath) : '';
        if (outputPath) {
            yield mkdirp(outputPath);
        }
        function build() {
            logger.log('Building GraphQL files...');
            return Promise.all([
                resolveGraphql(config.queries, queries, resolver, fileMap, FileType.Query, outputPath),
                resolveGraphql(config.mutations, mutations, resolver, fileMap, FileType.Mutation, outputPath),
            ]).then(() => {
                logger.success('Finished building GraphQL files');
            });
        }
        function watchFiles() {
            const ignored = ['node_modules', '.nuxt'];
            if (config.outputPath) {
                ignored.push(config.outputPath);
            }
            const filesWatcher = (options._filesWatcher = chokidar.watch('./**/*.graphql', {
                ignoreInitial: true,
                ignored,
            }));
            if (filesWatcher) {
                logger.info('Watching for query changes');
                filesWatcher.on('change', () => {
                    build();
                });
            }
        }
        this.nuxt.hook('build:done', () => {
            build().then(() => {
                if (options.debug) {
                    logger.info('Available queries and mutations:');
                    console.table(Array.from(fileMap.entries()).map(([_key, value]) => value));
                }
            });
            if (this.nuxt.options.dev) {
                watchFiles();
            }
        });
        this.addServerMiddleware({
            path: config.endpointNamespace,
            handler: serverMiddleware(config.graphqlServer, queries, mutations, config.server),
        });
    });
};
export default graphqlMiddleware;
